﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ConverterSlow
{
    public class BillingCycle : NotificationObject
    {
        public BillingCycle()
        { }

        public BillingCycle(int? id, string description, DateTime beginDate, string beginBy, DateTime? closeDate, string closeBy, DateTime? firstDateOfCycle, DateTime? lastDateOfCycle, DateTime? billingDataDate)
        {
            BillingCycleID = id;
            BillingCycleDescription = description;
            BeginDate = beginDate;
            BeginBy = beginBy;
            CloseDate = closeDate;
            CloseBy = closeBy;
            FirstDateOfCycle = firstDateOfCycle;
            LastDateOfCycle = lastDateOfCycle;
            BillingDataDate = billingDataDate;
        }

        private int? _billingCycleID;
        /// <summary>
        /// Billing Cycle ID
        /// </summary>
        public int? BillingCycleID
        {
            get { return _billingCycleID; }
            set
            {
                if (_billingCycleID != value)
                {
                    _billingCycleID = value;
                    FirePropertyChanged(() => BillingCycleID);
                }
            }
        }
        private string _billingCycleDescription;
        /// <summary>
        /// Billing Cycle description
        /// </summary>
        public string BillingCycleDescription
        {
            get { return _billingCycleDescription; }
            set
            {
                if (_billingCycleDescription != value)
                {
                    _billingCycleDescription = value;
                    FirePropertyChanged(() => BillingCycleDescription);
                }
            }
        }

        private DateTime _beginDate;
        /// <summary>
        /// Date of billing cycle begin
        /// </summary>
        public DateTime BeginDate
        {
            get { return _beginDate; }
            set
            {
                if (_beginDate != value)
                {
                    _beginDate = value;
                    FirePropertyChanged(() => BeginDate);
                }
            }
        }

        private string _beginBy;
        /// <summary>
        /// Who began the billing cycle
        /// </summary>
        public string BeginBy
        {
            get { return _beginBy; }
            set
            {
                if (_beginBy != value)
                {
                    _beginBy = value;
                    FirePropertyChanged(() => BeginBy);
                }
            }
        }

        private DateTime? _closeDate;
        /// <summary>
        /// Close date of billing cycle
        /// </summary>
        public DateTime? CloseDate
        {
            get { return _closeDate; }
            set
            {
                if (_closeDate != value)
                {
                    _closeDate = value;
                    FirePropertyChanged(() => CloseDate);
                }
            }
        }

        private string _closeBy;
        public string CloseBy
        {
            get { return _closeBy; }
            set
            {
                if (_closeBy != value)
                {
                    _closeBy = value;
                    FirePropertyChanged(() => CloseBy);
                }
            }
        }

        private DateTime? _firstDateOfCycle;
        /// <summary>
        /// First date of billing cycle
        /// </summary>
        public DateTime? FirstDateOfCycle
        {
            get { return _firstDateOfCycle; }
            set
            {
                if (_firstDateOfCycle != value)
                {
                    _firstDateOfCycle = value;
                    FirePropertyChanged(() => FirstDateOfCycle);
                }
            }
        }

        private DateTime? _lastDateOfCycle;
        /// <summary>
        /// Last date of billing cycle
        /// </summary>
        public DateTime? LastDateOfCycle
        {
            get { return _lastDateOfCycle; }
            set
            {
                if (_lastDateOfCycle != value)
                {
                    _lastDateOfCycle = value;
                    FirePropertyChanged(() => LastDateOfCycle);
                }
            }
        }

        private DateTime? _billingDataDate;
        /// <summary>
        /// Billing Data Date
        /// </summary>
        public DateTime? BillingDataDate
        {
            get { return _billingDataDate; }
            set
            {
                if (_billingDataDate != value)
                {
                    _billingDataDate = value;
                    FirePropertyChanged(() => BillingDataDate);
                }
            }
        }

        /// <summary>
        /// Overrides the Equals function so that Frequency objects don't have to be the exact same instance to be equal
        /// </summary>
        /// <param name="obj"></param>
        /// <returns></returns>
        public override bool Equals(object obj)
        {
            if (obj == null || !(obj is BillingCycle))
                return false;
            return ((BillingCycle)obj).BillingCycleID == this.BillingCycleID;
        }

        public static bool operator ==(BillingCycle a, BillingCycle b)
        {
            // If both are null, or both are same instance, return true.
            if (System.Object.ReferenceEquals(a, b))
            {
                return true;
            }

            // If one is null, but not both, return false.
            if (((object)a == null) || ((object)b == null))
            {
                return false;
            }

            // Return true if the fields match:
            return a.BillingCycleID == b.BillingCycleID;
        }

        public static bool operator !=(BillingCycle a, BillingCycle b)
        {
            return !(a == b);
        }

        /// <summary>
        /// Included to not violate equals objects must have same hash code
        /// </summary>
        /// <returns></returns>
        public override int GetHashCode()
        {
            return this.BillingCycleID.GetHashCode();
        }
    }
}

